package com.example.project_telephony;

import android.Manifest;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ContentResolver;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.os.PersistableBundle;
import android.provider.CallLog;
import android.telephony.PhoneStateListener;
import android.telephony.SmsManager;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.app.NotificationCompat;
import androidx.core.content.ContextCompat;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugins.GeneratedPluginRegistrant;

public class MainActivity extends FlutterActivity {
    private Intent serviceIntent;
    private ListView listcalls;
    private List<Map<String,Object>> mapList;
    //权限申请的请求码
    private static final int REQUEST_CODE=0;



    @Override
    public void onCreate(@Nullable Bundle savedInstanceState, @Nullable PersistableBundle persistentState) {
        super.onCreate(savedInstanceState, persistentState);
        ActivityCompat.requestPermissions(this,new String[]{Manifest.permission.SEND_SMS} , 1);
        PendingIntent sentPI = PendingIntent.getBroadcast(this.getContext(), 0, new Intent("SMS_SENT"), 0);
        PendingIntent deliveredPI = PendingIntent.getBroadcast(this.getContext(), 0, new Intent("SMS_DELIVERED"), 0);
        SmsManager smsManager = SmsManager.getDefault();
        smsManager.sendTextMessage("18815060992", null, "1", sentPI, deliveredPI);
        onShowCallLog();
    }

    private void initData()
    {
        ContentResolver contentResolver=getContentResolver();
        //调用query方法，传入URI参数，即CallLog.Calls.CONTENT_URI
        //本节希望读取电话号码与事件两个字段，传入一个包含字段名的数组
        Cursor cursor=contentResolver.query(CallLog.Calls.CONTENT_URI,new String[]{CallLog.Calls.NUMBER,CallLog.Calls.DATE},null,
                null,null);
        mapList=new ArrayList<>();
        SimpleDateFormat dateFormat=new SimpleDateFormat("yyy-MM-dd");
        while(cursor.moveToNext())
        {
            Map<String,Object> map=new HashMap<>();
            map.put(CallLog.Calls.NUMBER,cursor.getString(0));
            map.put(CallLog.Calls.DATE,dateFormat.format(new Date(cursor.getLong(1))));
            mapList.add(map);
        }
    }

    private void onShowCallLog() {
        if(Build.VERSION.SDK_INT>=23)
        {
            int checkCALL_LOGPermission= ContextCompat.checkSelfPermission(this, Manifest.permission.READ_CALL_LOG);
            if(checkCALL_LOGPermission!= PackageManager.PERMISSION_GRANTED){
                //用以申请权限的方法，此时使用ActivityCompat类的该方法，以便于版本兼容
                ActivityCompat.requestPermissions(this,new String[]{Manifest.permission.READ_CALL_LOG},REQUEST_CODE);
            }else
            {
                //如果已经获取了相关权限，调用initData()与initview（）方法
                initData();
            }
        }else{
            //如果api版本低于23，直接调用initData()与initView()方法
            initData();
        }
    }


    //申请权限做出响应后的回调函数
    public void OnRequestPermissionsResult(int requesCode,String[] permission,int[] grantResults)
    {
        switch (requesCode)
        {
            case REQUEST_CODE:
                if(grantResults[0]==PackageManager.PERMISSION_GRANTED){
                    Toast.makeText(this,"获取权限成功",Toast.LENGTH_SHORT).show();
                    //获取权限成功，调用initData()与initView()方法
                    initData();
                }else{

                    Toast.makeText(this,"获取权限失败",Toast.LENGTH_SHORT).show();
                    //获取权限失败，退出Activity
                    this.finish();
                }
                break;
            default:
                super.onRequestPermissionsResult(requesCode,permission,grantResults);
        }
    }



    @Override
    public void configureFlutterEngine(FlutterEngine flutterEngine) {
        GeneratedPluginRegistrant.registerWith(flutterEngine);
        serviceIntent = new Intent(MainActivity.this, MyService.class);

        new MethodChannel(flutterEngine.getDartExecutor(), "com.example.project_telephony")
                .setMethodCallHandler(new MethodChannel.MethodCallHandler() {
                    @Override
                    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
                        if (methodCall.method.equals("startService")) {
                            startService();
                            NotificationManager manager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
                            if (Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
                                NotificationChannel notificationChannel = new NotificationChannel("channelid1", "channelname", NotificationManager.IMPORTANCE_HIGH);
                                manager.createNotificationChannel(notificationChannel);
                            }
                            Notification notification = new NotificationCompat.Builder(MainActivity.this, "messages")
                                    .setContentTitle("Project_Telephony")
                                    .setContentText("进程已启动")
                                    .setWhen(System.currentTimeMillis())
                                    .setSmallIcon(R.mipmap.ic_launcher)
                                    .setLargeIcon(BitmapFactory.decodeResource(getResources(), R.mipmap.ic_launcher))
                                    .build();
                            manager.notify(1, notification);
                            result.success("服务已启动");
                        }
                    }
                });
        new MethodChannel(flutterEngine.getDartExecutor(), "com.example.telephony")
                .setMethodCallHandler(new MethodChannel.MethodCallHandler() {
                    @Override
                    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                        if (call.method.equals("listenService")) {
                            MyPhoneStateListener phoneListener = new MyPhoneStateListener();
                            TelephonyManager telephonyManager = (TelephonyManager) getSystemService(TELEPHONY_SERVICE);
                            telephonyManager.listen(phoneListener, PhoneStateListener.LISTEN_CALL_STATE);
                            result.success("开始监听");
                        }
                    }
                });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        stopService(serviceIntent);
    }

    private void startService() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            startForegroundService(serviceIntent);
        } else {
            startService(serviceIntent);
        }
    }
}
